/*   dir.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996,1997,1998,1999 by Christoph Schwarz
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "tweak.h"
#include "externs.h"
#include "perm.h"

int show_readmefile(void)
{ FILE* file;
  char  line[256];

  if (readme_file) if ((file = file_open(cwd,readme_file,"rt",0)) != NULL)
  { while (fgets(line,sizeof(line),file))
    { if (send_message(250,0,line,cwd) == -1)
      { fclose(file);
        return -1;
      }
    }
    fclose(file);
  }
  return 0;
}

int find_matching_dir(char* path, char* wildcard)
{ struct dirent* d;
  DIR*           dir;
  char*          name;
  struct stat    st;

  dir = opendir(strcmp(path,"/") ? path+1 : ".");
  if (!dir) return 0;
  if (strcmp(path,"/")) strcat(path,"/");
  name = path + strlen(path);
  while ((d = readdir(dir)) != NULL)
  { strcpy(name,d->d_name);
    if (strcmp(name,".") && strcmp(name,"..") && !stat(path+1,&st) &&
      S_ISDIR(st.st_mode))
    { if (wildcard_matching(wildcard,name,0))
      { closedir(dir);
        return 1;
      }
    }
  }
  closedir(dir);
  return 0;
}
char* is_shortcut(char *shortcutpath)
{ int n=0;
  int found=0;
 
  for(n=0;n<dshortcut_count;n++)
  {
     if(!strcasecmp(shortcutpath,dshortcut[n].shortname))
     {
        found=1;
        break;
     }
  }
  if(found)
     return (char *)dshortcut[n].longname;
  return NULL;
}
int cmd_cwd(char** p, int n)
{ char* dir;
  char* name;
  char* shortcutpath;
  char  wd[256];

  if (n != 1) return send_reply(501,"Syntax: CWD <directory>");
  if((shortcutpath=is_shortcut(*p))!=NULL)
     strcpy(*p,shortcutpath);
  n = add_dir(cwd,*p,wd,sizeof(wd));
  client_log_start(LG_CD,"CD %s",n == -1 ? *p : wd);
  if (n != -1)
  { n = permission_ok(*p,P_CHDIR);
    if (n == -1) if (errno == ENOENT)
    { split_dir(wd,&dir,&name,0);
      strcpy(*p,name);
      if (!dir) strcpy(wd,"/");
      if (find_matching_dir(wd,*p)) n = permission_ok(wd,P_CHDIR); else
      { n = -1;
        errno = ENOENT;
      }
    }
  }
  if (n == -1) return send_error(550,"chdir",LG_CD);
  strcpy(cwd,wd);
  client_log_finish(LG_CD,NULL);
  if (show_readmefile() == -1) return -1;
  return send_message(250,1,chdir_msg,wd);
}
 
int cmd_cdup(char** p, int n)
{ char wd[256];

  if (n) return send_reply(501,"Syntax: CDUP");
  strcpy(*p,"..");
  n = add_dir(cwd,*p,wd,sizeof(wd));
  client_log_start(LG_CD,"CD %s",n == -1 ? *p : wd);
  if ((n == -1) || (permission_ok(*p,P_CHDIR) == -1))
    return send_error(550,"cdup",LG_CD);
  strcpy(cwd,wd);
  client_log_finish(LG_CD,NULL);
  if (show_readmefile() == -1) return -1;
  return send_message(250,1,cdup_msg,cwd);
}

int cmd_mkd(char** p, int n)
{ char* dir;
  char* name;
  int   ok = 0;
  char  wd[256];

  if (n != 1) return send_reply(501,"Syntax: MKD <directory>");
  ok = add_dir(cwd,*p,wd,sizeof(wd));
  client_activity("MKDIR %s",ok == -1 ? *p : wd);
  client_log_start(LG_MD,"MKDIR %s",ok == -1 ? *p : wd);
  if (ok != -1)
  { split_dir(wd,&dir,&name,0);
    convert_filename(name,fconv_mode >> 4);
    ok = permission_ok(dir,P_MKDIR);
    if (ok != -1)
    { *(name-1) = '/';
      ok = mkdir(wd+1,0777);
      if (ok != -1)
      { ok = write_permission_file(wd,usr.uid,usr.gid,umask_value);
        if ((ok == -1) && (errno == EACCES)) ok = 0;
      }
    }
  }
  if (ok == -1) return send_error(550,"mkdir",LG_MD);
  client_log_finish(LG_MD,NULL);
  return send_message(257,1,mkdir_msg,*p);
}

int directory_empty(char* wd)
{ DIR*           d;
  struct dirent* de;

  if (!strcmp(wd,"/"))
  { errno = EACCES;
    return -1;
  }
  d = opendir(wd+1);
  if (!d) return -1;
  de = readdir(d);
  while (de)
  { if (strcmp(de->d_name,".") && strcmp(de->d_name,"..") &&
        strcmp(de->d_name,permission_file))
    { if (!readme_file || strcmp(de->d_name,readme_file))
      { closedir(d);
        return 0;
      }
    }
    de = readdir(d);
  }
  closedir(d);
  return 1;
}

int cmd_rmd(char** p, int n)
{ int   ok = 0;
  char  wd[256];

  if (n != 1) return send_reply(501,"Syntax: RMD <directory>");
  ok = add_dir(cwd,*p,wd,sizeof(wd));
  client_activity("RMDIR %s",ok == -1 ? *p : wd);
  client_log_start(LG_RD,"RMDIR %s",ok == -1 ? *p : wd);
  if (ok != -1)
  { if (!strcmp(wd,cwd))
      return send_reply(553,"Cannot remove current directory.");
    ok = permission_ok(wd,P_CHMOD);
    if (ok != -1)
    { ok = directory_empty(wd);
      if (ok != -1)
      { if (!ok) return send_reply(553,"Directory is not empty.");
        ok = remove_permission_file(wd);
        if (ok != -1) ok = rmdir(wd+1);
      }
    }
  }
  if (ok == -1) return send_error(553,"rmdir",LG_RD);
  client_log_finish(LG_RD,NULL);
  return send_reply(253,"RMD command successful.");
}

int cmd_pwd(char** p, int n)
{ if (n) return send_reply(501,"Syntax: PWD");
  return send_message(257,1,"\"%s\" is current directory.",cwd);
}

/* EOF */
