/*   ff.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996,1997,1998,1999  Christoph Schwarz
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include <sys/types.h>
#include <ctype.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "tweak.h"
#include "externs.h"

int cmd_site_ff_syntax(int rno)
{ return send_reply(rno,
    "Syntax: SITE FF <wildcard>\n"
    "Finds all files matching <wildcard>. Search starts in the current\n"
    "directory.");
}

int cmd_site_ff_r(char* path, char* wildcard)
{ struct dirent* d;
  DIR*           dir;
  int            m;
  int            n = 0;
  char*          name;
  struct stat    st;

  dir = opendir(path);
  if (!dir) return 0;
  strcat(path,"/");
  name = path + strlen(path);
  while ((d = readdir(dir)) != NULL)
  { strcpy(name,d->d_name);
    if (strcmp(d->d_name,".") && strcmp(d->d_name,"..") && !stat(path,&st))
    { if (wildcard_matching(wildcard,name,0))
      { n++;
        if (send_message(221,0,path+1,NULL) == -1) return -1;
      }
      if (S_ISDIR(st.st_mode))
      { m = cmd_site_ff_r(path,wildcard);
        if (m == -1) return -1;
        n += m;
      }
    }
  }
  closedir(dir);
  return n;
}

int cmd_site_ff(char** p, int n)
{ char path[512];

  if (n != 1) return cmd_site_ff_syntax(501);
  client_activity("FF %s",*p);
  strcpy(path,".");
  if (*ff_path)
  { if (strcmp(ff_path,"/")) strcpy(path+1,ff_path);
  } else
  { if (strcmp(cwd,"/")) strcpy(path+1,cwd);
  }
  n = cmd_site_ff_r(path,*p);
  if (n == -1) return -1;
  if (!n) return send_reply(221,"No matches found.");
  sprintf(path,"%i",n);
  return send_message(221,1,"%s match(es) found.",path);
}

void convert_filename(char* filename, int mode)
{ char* name;

  if (!mode) return;
  name = filename + strlen(filename);
  while ((name > filename) && (*(name-1) != '/')) name--;
  filename = name;
  switch (mode & 0x03)
  { case 1 :
      while (*name)
      { *name = tolower(*name);
        name++;
      }
      break;
    case 2 :
      while (*name)
      { *name = toupper(*name);
        name++;
      }
      break;
    case 3 :
      *name = toupper(*name);
      break;
  }
  if (mode & 0x04) while (*filename)
  { if (isspace(*filename)) *filename = '_';
    filename++;
  }
}

int find_dupe_r(char* path, char* filename)
{ struct dirent* d;
  DIR*           dir;
  int            n = 0;
  char*          name;
  struct stat    st;

  dir = opendir(path);
  if (!dir) return 0;
  strcat(path,"/");
  name = path + strlen(path);
  while (!n && ((d = readdir(dir)) != NULL))
  { strcpy(name,d->d_name);
    if (strcmp(d->d_name,".") && strcmp(d->d_name,"..") && !stat(path,&st))
    { if (!strcmp(name,filename))
      { closedir(dir);
        return 1;
      }
      if (S_ISDIR(st.st_mode)) n += find_dupe_r(path,filename);
    }
  }
  closedir(dir);
  return n;
}

int find_dupe(char* filename, char* dir)
{ int  n = 0;
  char path[512];

  while (n < dupechk_size)
  { if (wildcard_matching(dupechk+n,filename,0))
    { strcpy(path,".");
      if (*dupe_path)
      { if (strcmp(dupe_path,"/")) strcpy(path+1,dupe_path);
      }
      n = find_dupe_r(path,filename);
      strcpy(dir,path+1);
      return n;
    }
    n += strlen(dupechk+n) + 1;
  }
  return 0;
}
 
/* EOF */
