/*   login.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996,1997,1998,1999 by Christoph Schwarz
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tweak.h"
#include "externs.h"

int cmd_user(char** p, int n)
{ int   anon_login;
  char  name[sizeof(usr.name)];
  char  str[128];
  int   kick = 0;

  if (*usr.name)
    return send_reply(530,"Cannot change user id, log out first.");
  if (n != 1) return send_reply(501,"User name required.");
  if (login_kick && **p=='!') kick = 1;
  if (strlen(*p) >= (sizeof(name) + kick))
    return send_reply(501,"User name too long.");
  strcpy(name,(*p)+kick);
  *usr.name = '\0';
  printf("UAN %s\n",name);
  fgets(str, sizeof(str), stdin);
  anon_login = (*str == '+');
  anonymous = (*(str+1) == 'a');
  if (anonymous)
  { if (!anon_login)
    { client_log_start(LG_LOGIN,"LOGIN by %s",name);
      client_log_finish(LG_LOGIN," FAILED: Guest login disabled");
      return send_reply(530,"Guest login disabled.");
    }
    strcpy(str,"Guest login initiated, send E-Mail address as password.");
  } else sprintf(str,"Password required for %s.",name);
  if (send_reply(331,str) == -1) return -1;
  switch (read_ftp_command(str,sizeof(str),p,4,&n))
  { case -1 : return -1;
    case  0 : break;
    default : return 0;
  }
  if (strcasecmp(*p,"PASS"))
    return send_reply(503,"Please login with USER and PASS.");
  if (n > 2) return send_reply(501,"Syntax: PASS <password>");
  if (n == 1) printf("ULP %i %s ''\n",kick,name); else
    printf("ULP %i %s %s\n",kick,name,*(p+1));
  if (!fgets(str,sizeof(str),stdin)) strcpy(str,"--");
  if (*str == '-')
  { retries++;
    if (login_retries && (retries >= login_retries))
    { quit(CL_NORMAL);
      client_log_start(LG_LOGIN,"LOGIN by %s",name);
      client_log_finish(LG_LOGIN," FAILED: Too many login failures");
      return send_reply(421,"Too many login failures, connection closed.");
    }
    switch(tolower(*(str+1)))
    { case '-' : return send_reply(530,"Pipe error, login failed.");
      case 'a' :
        return send_reply(530,"You are already logged in maximum times.");
      case 'c' : quit(CL_NORMAL);
        return send_reply(421,"Server is temporary closed.");
      case 'd' : quit(CL_NORMAL);
        return send_reply(421,"Your account has been disabled.");
      case 'i' :
        return send_reply(530,"You are not allowed to log in from this "
          "Ident@IP.");
      case 'n' :
        return send_reply(530,"Anonymous login is disabled.");
      case 'p' :
        return send_reply(530,"Login incorrect.");
      case 't' : quit(CL_NORMAL);
        get_message(anonymous ? "MTA" : "MTU",&message,&msg_len,&msg_size);
        sleep(5);
        return send_message(421,1,message,usr.name);
      case 'u' : quit(CL_NORMAL);
        return send_reply(421,"No more connections from your IP/domain.");
      default  :
        return send_reply(530,"Login failed, reason unknown.");
    }
  }
  strcpy(usr.name,name);
  sscanf(str+1,"%li %li %li %li %li %i %i %li %li %li %li %li %li %li %li %li"
    " %li %li",&usr.uid,&usr.gid,&usr.kb_downloaded,&usr.kb_uploaded,
    &usr.last_login,&timeout,&usr.limit,&usr.credit,usr.gids,usr.gids+1,
    usr.gids+2,usr.gids+3,usr.gids+4,usr.gids+5,usr.gids+6,usr.gids+7,
    &usr.class,&umask_value);
  if (!get_message("MWC",&message,&msg_len,&msg_size)) return -1;
  if (anonymous) num_anon++; else if (!superuser()) num_users++;
  if (send_message(230,1,message,usr.name) == -1) return -1;
  logged_in = 1;
  return 0;
}

int cmd_pass(char** p, int n)
{ return send_reply(503,"Login with USER first.");
}

int cmd_acct(char** p, int n)
{ return send_reply(502,"ACCT command not implemented.");
}

int cmd_quit(char** p, int n)
{ quit(CL_NORMAL);
  usr.kb_downloaded += (bytes_downloaded / 1024 + mbytes_downloaded * 1024);
  usr.kb_uploaded += (bytes_uploaded / 1024 + mbytes_uploaded * 1024);
  if (!get_message("MGB",&message,&msg_len,&msg_size))
    return send_reply(221,"Goodbye.");
  if (!*usr.name) 
  { strcpy(usr.name,"*UNKNOWN*");
    return send_message(221,1,"Goodbye.",usr.name);
  } else return send_message(221,1,message,usr.name);
}

/* EOF */
